////////////////////////////////////////////////////////////////////////////////
/// \file snapping.cpp 
///
/// Example code of 3D cursor snapping to geometry deformed on the GPU.
///
/// (c)2005 Harlen Batagelo, harlen@dca.fee.unicamp.br
////////////////////////////////////////////////////////////////////////////////

#include "dxstdafx.h"
#include "d3dfile.h"
#include "d3dfont.h"
#include "resource.h"

//#define DEBUG_VS   // Uncomment this line to debug vertex shaders 
//#define DEBUG_PS   // Uncomment this line to debug pixel shaders 

#include "snapping.h"

bool g_bShowCursor = true;

D3DVERTEXELEMENT9 g_VDeclSimple[] = 
{
{ 0,  0, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },
{ 0, 12, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL,   0 },
{ 0, 24, D3DDECLTYPE_FLOAT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },
D3DDECL_END()
};

D3DVERTEXELEMENT9 g_VDeclActual[] = 
{
{ 0,  0, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },
{ 0, 12, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL,   0 },
{ 0, 24, D3DDECLTYPE_FLOAT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },
{ 0, 32, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TANGENT,  0 },
{ 0, 44, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_BINORMAL, 0 },
D3DDECL_END()
};

D3DVERTEXELEMENT9 g_VDeclInteract[] = 
{
{ 0,  0, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },
{ 0, 12, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL,   0 },
{ 0, 24, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },
{ 0, 36, D3DDECLTYPE_FLOAT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 1 },
{ 0, 44, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TANGENT,  0 },
{ 0, 56, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_BINORMAL, 0 },
D3DDECL_END()
};

////////////////////////////////////////////////////////////////////////////////
/// \brief Entry point to the program.
///
/// Initializes everything, and goes into a message-processing loop. Idle time 
/// is used to render the scene.
///
/// \return Application exit code.
////////////////////////////////////////////////////////////////////////////////
INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT )
{
    CApp pApp;

    InitCommonControls( );
    if( FAILED( pApp.Create( hInst ) ) )
        return 0;

    return pApp.Run();
} // WinMain


////////////////////////////////////////////////////////////////////////////////
/// \brief Application class constructor.
///
/// Sets the initial states of variables and attributes for the application.
///
/// \return None.
////////////////////////////////////////////////////////////////////////////////
CApp::CApp( )
{
    m_strWindowTitle = _T("Snapping to Geometry Deformed on the GPU");
    m_d3dEnumeration.AppUsesDepthBuffer = true;
    m_bShowCursorWhenFullscreen         = true;
    m_dwCreationWidth                   = 800;
    m_dwCreationHeight                  = 600;

	m_fZNear = 0.1f;
	m_fZFar  = 50.0f;
	m_fZDist = 10.0f;

	m_fDeform1 = 10.0f;
	m_fDeform2 = 10.0f;

	m_pTex = NULL;
	m_pBumpTex = NULL;

    m_hWorld               = NULL;
	m_hView                = NULL;
    m_hWorldViewProjection = NULL;
	m_hDiffuse             = NULL;
	m_hAmbient             = NULL;
	m_hDeform1             = NULL;
	m_hDeform2             = NULL;

	m_pTriadCenter         = NULL;
	m_pTriadAxis           = NULL;

    m_pFont                = new CD3DFont( _T("Verdana"), 13, D3DFONT_BOLD );
    m_pFontSmall           = new CD3DFont( _T("Verdana"), 10, D3DFONT_BOLD );
    m_pObject              = new CD3DMesh();
	m_bIntersected         = false;

	m_pVB                  = NULL;
	m_pIB                  = NULL;
	m_pVBNI                = NULL;

	m_dwNumVerts           = 0;
	m_dwNumFaces           = 0;
    m_pVBPicked            = NULL;
    m_bShowHelp            = false;
	m_bShowNormalData      = false;
	m_bShowDepthData       = false;
	m_bShowFaceID          = false;
	m_bShowBaryData        = false;
	m_bShowSnapTriangle    = false;
	m_bDrawTriad           = true;

	m_pRTReadSurf          = NULL;
	m_pBackBufferSurf      = NULL;
    m_pRTTexture           = NULL;
	m_pRTSurface           = NULL;

	m_pVertexDeclInteract  = NULL;
	m_pVertexDeclActual    = NULL;
	m_pVertexDeclSimple    = NULL;

    m_pEffect = NULL;
} // CApp:CApp


////////////////////////////////////////////////////////////////////////////////
/// \brief One time initialization function.
///
/// Called during initial app startup, this function performs all the permanent 
/// initialization.
////////////////////////////////////////////////////////////////////////////////
HRESULT CApp::OneTimeSceneInit()
{
    return S_OK;
} // CApp::OneTimeSceneInit


////////////////////////////////////////////////////////////////////////////////
/// \brief Updates the animation of the scene.
///
/// Called once per frame, the call is the entry point for animating the scene.
///
/// \return \c S_OK on success.
////////////////////////////////////////////////////////////////////////////////
HRESULT CApp::FrameMove()
{
    D3DXVECTOR3 vLookatPt = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
    D3DXVECTOR3 vUpVec    = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );
    D3DXVECTOR3 vFromPt;

	vFromPt.x =   0.0f;
	vFromPt.y =   0.0f;
	vFromPt.z = -27.0f;

    D3DXMATRIXA16 matView;
    D3DXMatrixLookAtLH( &matView, &vFromPt, &vLookatPt, &vUpVec );
    m_pd3dDevice->SetTransform( D3DTS_VIEW, &matView );

	// Animate deformation
	static bool bInc1 = true;
	static bool bInc2 = false;

	if( bInc1 )
	{
		m_fDeform1 += m_fElapsedTime*3.5f;
		if( m_fDeform1 >= 15.0f )
			bInc1 = false;
	}
	else
	{
		m_fDeform1 -= m_fElapsedTime*2.5f;
		if( m_fDeform1 <= -10.0f )
			bInc1 = true;
	}

	if( bInc2 )
	{
		m_fDeform2 += m_fElapsedTime*3.0f;
		if( m_fDeform2 >= 15.0f )
			bInc2 = false;
	}
	else
	{
		m_fDeform2 -= m_fElapsedTime*2.5f;
		if( m_fDeform2 <= -15.0f )
			bInc2 = true;
	}

    return S_OK;
} // CApp::FrameMove


////////////////////////////////////////////////////////////////////////////////
/// \brief Sets up the snap matrix.
///
/// Sets up the viewport and transformation matrices to that only the pxel 
/// with the given coordinates will be visible.
///
/// \param iX X screen coordinate of the pixel to isolate.
/// \param iY Y screen coordinate of the pixel to isolate.
///
/// \return None.
////////////////////////////////////////////////////////////////////////////////
void CApp::SetSnapMatrix( int iX, int iY )
{
	// Get viewport settings
	D3DVIEWPORT9 d3dVP;
	m_pd3dDevice->GetViewport ( &d3dVP );

	// Unproject mouse position for computing the top-left frustum location
	// for snapping
	D3DXVECTOR3 vUnProj1;
	D3DXVECTOR3 vMouse = D3DXVECTOR3( (float) iX, (float) iY, 0.0f );
	D3DXVec3Unproject( &vUnProj1, &vMouse, &d3dVP, &m_mP, &m_mV, &m_mW );

	// Transform unprojected point from object space to camera space
	D3DXVECTOR4 vUnProj4_1;
	D3DXVec3Transform( &vUnProj4_1, &vUnProj1, &m_mWV );

	// Unproject mouse position for computing the bottom-right frustum location
	// for snapping
	D3DXVECTOR3 vUnProj2;
	vMouse = D3DXVECTOR3( (float) iX + 1.0f , (float) iY + 1.0f , 0.0f );
	D3DXVec3Unproject( &vUnProj2, &vMouse, &d3dVP, &m_mP, &m_mV, &m_mW );

	// Transform unprojected point from object space to camera space
	D3DXVECTOR4 vUnProj4_2;
	D3DXVec3Transform( &vUnProj4_2, &vUnProj2, &m_mWV );

	// Set snapping matrix
	D3DXMATRIXA16 mNewProj;
	D3DXMatrixPerspectiveOffCenterLH( &mNewProj, 
		                              vUnProj4_1.x, // left
		                              vUnProj4_2.x, // right
		                              vUnProj4_2.y, // bottom
		                              vUnProj4_1.y, // top
		                              m_fZNear, m_fZFar );
	m_pd3dDevice->SetTransform( D3DTS_PROJECTION, &mNewProj  );

	// Decrease viewport size for snapping
	d3dVP.X     = d3dVP.Y = 0;
	d3dVP.Width = d3dVP.Height = 1;
	d3dVP.MinZ  = 0.0f;
	d3dVP.MaxZ  = 1.0f;
	m_pd3dDevice->SetViewport( &d3dVP );
} // CApp::SetSnapMatrix


////////////////////////////////////////////////////////////////////////////////
/// \brief Sets up the object of interest.
///
/// Called for centering the object in world coordinates and changing its 
/// scaling factor to a suitable size.
///
/// \return \c S_OK on success.
////////////////////////////////////////////////////////////////////////////////
HRESULT CApp::SetUpObject( )
{
	HRESULT hrResult = S_OK;

	hrResult = m_pEffect->SetTexture( _T("Tex0"), m_pTex );
	if( FAILED( hrResult ) ) return hrResult;
	hrResult = m_pEffect->SetTexture( _T("Tex1"), m_pBumpTex );
	if( FAILED( hrResult ) ) return hrResult;

	// Center and scale object to a suitable location/size
	if( !m_bObjSetUp )
	{
		LPD3DXMESH pMesh = m_pObject->GetSysMemMesh( );
		if( pMesh )
		{
			// Create mesh clone
			LPD3DXMESH pCloneMesh;
			hrResult = pMesh->CloneMesh( D3DPOOL_SYSTEMMEM, g_VDeclInteract, 
										 m_pd3dDevice, &pCloneMesh );
			if( FAILED( hrResult ) ) return hrResult;

			// Copy mesh to clone mesh
			LPDIRECT3DVERTEXBUFFER9 pVB, pVBClone;
			LPDIRECT3DINDEXBUFFER9 pIB, pIBClone;

			// Get pointer to the vertex buffer
			hrResult = pMesh->GetVertexBuffer( &pVB );
			if( FAILED( hrResult ) )
				return hrResult;
			hrResult = pCloneMesh->GetVertexBuffer( &pVBClone );
			if( FAILED( hrResult ) )
				return hrResult;

			// Get pointer to the index buffer
			hrResult = pMesh->GetIndexBuffer( &pIB );
			if( FAILED( hrResult ) )
				return hrResult;
			hrResult = pCloneMesh->GetIndexBuffer( &pIBClone );
			if( FAILED( hrResult ) )
				return hrResult;

			// Lock vertex buffer
			D3DVERTEX_SIMPLE* pVertices;
			hrResult = pVB->Lock( 0, 0, (void**)&pVertices, 0 );
			if( FAILED( hrResult ) )
				return hrResult;
			D3DVERTEX_INTERACT* pVerticesClone;
			hrResult = pVBClone->Lock( 0, 0, (void**)&pVerticesClone, 0 );
			if( FAILED( hrResult ) )
				return hrResult;

			// Lock index buffer
			WORD* pIndices;
			hrResult = pIB->Lock( 0, 0, (void**)&pIndices, 0 );
			if( FAILED( hrResult ) )
				return hrResult;
			WORD* pIndicesClone;
			hrResult = pIBClone->Lock( 0, 0, (void**)&pIndicesClone, 0 );
			if( FAILED( hrResult ) )
				return hrResult;

			// Compute what I need here
			for( DWORD dwCt = 0; dwCt < pMesh->GetNumVertices( ); dwCt++ )
			{
				pVerticesClone[dwCt].vP = pVertices[dwCt].vP;
				pVerticesClone[dwCt].vN = pVertices[dwCt].vN;
				pVerticesClone[dwCt].fU = pVertices[dwCt].fU;
				pVerticesClone[dwCt].fV = pVertices[dwCt].fV;
				pVerticesClone[dwCt].vT = D3DXVECTOR3( 0.0, 0.0, 0.0 );
				pVerticesClone[dwCt].vB = D3DXVECTOR3( 0.0, 0.0, 0.0 );
			}

			for( DWORD dwCt = 0; dwCt < pMesh->GetNumFaces( ); dwCt++ )
			{
				WORD* iThisTri = &pIndicesClone[3*dwCt];

				// Get vertices of this face
				D3DVERTEX_INTERACT v1 = pVerticesClone[iThisTri[0]];
				D3DVERTEX_INTERACT v2 = pVerticesClone[iThisTri[1]];
				D3DVERTEX_INTERACT v3 = pVerticesClone[iThisTri[2]];

				//
				// Calculate tangent and binormal
				//

				// For the plane equation, we need to compute two sides of the 
				// triangle
				D3DXVECTOR3 vSide1 = 
					D3DXVECTOR3( v2.vP.x-v1.vP.x, v2.fU-v1.fU,	v2.fV-v1.fV );
				D3DXVECTOR3 vSide2 = 
					D3DXVECTOR3( v3.vP.x-v1.vP.x, v3.fU-v1.fU, v3.fV-v1.fV );

				// The cross product yields the normal to the two side vectors
				// The normal provides the actual values in the plane equation
				D3DXVECTOR3 vCrossProduct;
				D3DXVec3Cross( &vCrossProduct, &vSide1, &vSide2 );

				D3DXVECTOR3 vU, vV;
				// Find the x component of each texture gradient by dividing the
				// appropriate component by the x component
				vU.x = -vCrossProduct.y / vCrossProduct.x;
				vV.x = -vCrossProduct.z / vCrossProduct.x;

				// Repeat the same sequence of operations for the y component of
				// the U and V vectors
				vSide1.x = v2.vP.y - v1.vP.y;
				vSide2.x = v3.vP.y - v1.vP.y;
				D3DXVec3Cross( &vCrossProduct, &vSide1, &vSide2 );

				vU.y = -vCrossProduct.y / vCrossProduct.x;
				vV.y = -vCrossProduct.z / vCrossProduct.x;

				// Repeat the sequence one last time for the z component of the
				// two basis vectors
				vSide1.x = v2.vP.z - v1.vP.z;
				vSide2.x = v3.vP.z - v1.vP.z;
				D3DXVec3Cross( &vCrossProduct, &vSide1, &vSide2 );

				vU.z = -vCrossProduct.y / vCrossProduct.x;
				vV.z = -vCrossProduct.z / vCrossProduct.x;

				// Normalize the two vectors
				D3DXVec3Normalize( &vU, &vU );
				D3DXVec3Normalize( &vV, &vV );

				// Get the cross product. This is the normal vector to the
				// two gradients
				D3DXVECTOR3 vUxV;
				D3DXVec3Cross( &vUxV, &vU, &vV );

				// The basis vector normal should be pointing in roughly the 
				// same direction as the vertex normal. If the basis normal is 
				// facing in the opposite direction, we should flip the 
				// direction by negating the vector
				if( D3DXVec3Dot( &vUxV, 
					&D3DXVECTOR3( v1.vN.x, v1.vN.y, v1.vN.z ) ) < 0.0f )
				{
					vUxV = -vUxV;
				}

				// Now that we have all three components, write the vectors to
				// the three vertices.

				// Check if we need to average the tangents and binormals
				bool bAverage1 = true;
				if( pVerticesClone[iThisTri[0]].vT.x == 99.9f )
				{
					pVerticesClone[iThisTri[0]].vT.x = 0.0f;
					bAverage1 = false;
				}

				bool bAverage2 = true;
				if( pVerticesClone[iThisTri[1]].vT.x == 99.9f )
				{
					pVerticesClone[iThisTri[1]].vT.x = 0.0f;
					bAverage2 = false;
				}

				bool bAverage3 = true;
				if( pVerticesClone[iThisTri[2]].vT.x == 99.9f )
				{
					pVerticesClone[iThisTri[2]].vT.x = 0.0f;
					bAverage3 = false;
				}

				// Update tangents and binormals
				pVerticesClone[iThisTri[0]].vT += vU;
				pVerticesClone[iThisTri[0]].vB += vV;
				pVerticesClone[iThisTri[1]].vT += vU;
				pVerticesClone[iThisTri[1]].vB += vV;
				pVerticesClone[iThisTri[2]].vT += vU;
				pVerticesClone[iThisTri[2]].vB += vV;

				if( bAverage1 )
				{
					D3DXVec3Normalize( &pVerticesClone[iThisTri[0]].vT, 
									   &pVerticesClone[iThisTri[0]].vT );
					D3DXVec3Normalize( &pVerticesClone[iThisTri[0]].vB, 
									   &pVerticesClone[iThisTri[0]].vB );
				}

				if( bAverage2 )
				{
					D3DXVec3Normalize( &pVerticesClone[iThisTri[1]].vT, 
									   &pVerticesClone[iThisTri[1]].vT );
					D3DXVec3Normalize( &pVerticesClone[iThisTri[1]].vB, 
									   &pVerticesClone[iThisTri[1]].vB );
				}

				if( bAverage3 )
				{
					D3DXVec3Normalize( &pVerticesClone[iThisTri[2]].vT, 
									   &pVerticesClone[iThisTri[2]].vT );
					D3DXVec3Normalize( &pVerticesClone[iThisTri[2]].vB, 
									   &pVerticesClone[iThisTri[2]].vB );
				}
			}

			pVB->Unlock( );
			pIB->Unlock( );
			pVB->Release( );
			pIB->Release( );

			// Compute bounding box
			D3DXVECTOR3 pMin, pMax, pCenter;
			hrResult = D3DXComputeBoundingBox( &pVertices[0].vP,
				pMesh->GetNumVertices( ),
				pMesh->GetNumBytesPerVertex( ),
				&pMin, &pMax );
			if( FAILED( hrResult ) )
				return hrResult;

			pCenter = (pMax + pMin) / 2.0f;
			float fScaleFactor = (6.0f / (pMax.y - pMin.y));

			for( DWORD dwCt = 0; dwCt < pCloneMesh->GetNumVertices( ); dwCt++ )
			{
				// Center object
				pVerticesClone[dwCt].vP -= pCenter; 
				// Scale to a suitable size
				pVerticesClone[dwCt].vP *= fScaleFactor; 
			}

			m_dwNumVerts = pCloneMesh->GetNumVertices( );
			m_dwNumFaces = pCloneMesh->GetNumFaces( );

			// Create indexed version of the cloned object
			SAFE_RELEASE( m_pVB );
			if( FAILED( m_pd3dDevice->CreateVertexBuffer( 
				m_dwNumVerts * sizeof(D3DVERTEX_ACTUAL),
				D3DUSAGE_WRITEONLY, D3DVERTEX_ACTUAL::FVF, 
				D3DPOOL_MANAGED, &m_pVB, NULL ) ) )
			{
				return E_FAIL;
			}

			SAFE_RELEASE( m_pIB );
			hrResult = pCloneMesh->GetIndexBuffer( &m_pIB );
			if( FAILED( hrResult ) )
				return hrResult;

			// Lock indexed VB
			D3DVERTEX_ACTUAL* pIVB;
			m_pVB->Lock( 0, 0, (void**)&pIVB, 0 );

			// Fill indexed VB
			for( DWORD dwVert = 0; dwVert < m_dwNumVerts; dwVert++ )
			{
				pIVB[dwVert].vP = pVerticesClone[dwVert].vP;
				pIVB[dwVert].vN = pVerticesClone[dwVert].vN;
				pIVB[dwVert].fU = pVerticesClone[dwVert].fU;
				pIVB[dwVert].fV = pVerticesClone[dwVert].fV;
				pIVB[dwVert].vT = pVerticesClone[dwVert].vT;
				pIVB[dwVert].vB = pVerticesClone[dwVert].vB;
			}

			m_pVB->Unlock( );

			// Create non-indexed version of the cloned object
			SAFE_RELEASE( m_pVBNI );
			if( FAILED( m_pd3dDevice->CreateVertexBuffer( 
				m_dwNumFaces * 3 * sizeof(D3DVERTEX_INTERACT),
				D3DUSAGE_WRITEONLY, D3DVERTEX_INTERACT::FVF, 
				D3DPOOL_MANAGED, &m_pVBNI, NULL ) ) )
			{
				return E_FAIL;
			}

			// Lock non-indexed VB
			D3DVERTEX_INTERACT* pVBNI;
			m_pVBNI->Lock( 0, 0, (void**)&pVBNI, 0 );

			// Fill non-indexed VB
			for( DWORD dwFaces = 0; dwFaces < m_dwNumFaces; dwFaces++ )
			{
				WORD* iThisTri = &pIndicesClone[3*dwFaces];

				// Get vertices of this face
				D3DVERTEX_INTERACT v1 = pVerticesClone[iThisTri[0]];
				D3DVERTEX_INTERACT v2 = pVerticesClone[iThisTri[1]];
				D3DVERTEX_INTERACT v3 = pVerticesClone[iThisTri[2]];

				// Now add them no the non-indexed VB
				pVBNI[(dwFaces * 3) + 0].vN     = v1.vN;
				pVBNI[(dwFaces * 3) + 0].vP     = v1.vP;
				pVBNI[(dwFaces * 3) + 0].fU     = v1.fU;
				pVBNI[(dwFaces * 3) + 0].fV     = v1.fV;
				pVBNI[(dwFaces * 3) + 0].vT     = v1.vT;
				pVBNI[(dwFaces * 3) + 0].vB     = v1.vB;
				pVBNI[(dwFaces * 3) + 0].fFace  = (FLOAT) dwFaces;
				pVBNI[(dwFaces * 3) + 0].fBary1 = 0.0;
				pVBNI[(dwFaces * 3) + 0].fBary2 = 0.0;

				pVBNI[(dwFaces * 3) + 1].vN     = v2.vN;
				pVBNI[(dwFaces * 3) + 1].vP     = v2.vP;
				pVBNI[(dwFaces * 3) + 1].fU     = v2.fU;
				pVBNI[(dwFaces * 3) + 1].fV     = v2.fV;
				pVBNI[(dwFaces * 3) + 1].vT     = v2.vT;
				pVBNI[(dwFaces * 3) + 1].vB     = v2.vB;
				pVBNI[(dwFaces * 3) + 1].fFace  = (FLOAT) dwFaces;
				pVBNI[(dwFaces * 3) + 1].fBary1 = 1.0;
				pVBNI[(dwFaces * 3) + 1].fBary2 = 0.0;

				pVBNI[(dwFaces * 3) + 2].vN     = v3.vN;
				pVBNI[(dwFaces * 3) + 2].vP     = v3.vP;
				pVBNI[(dwFaces * 3) + 2].fU     = v3.fU;
				pVBNI[(dwFaces * 3) + 2].fV     = v3.fV;
				pVBNI[(dwFaces * 3) + 2].vT     = v3.vT;
				pVBNI[(dwFaces * 3) + 2].vB     = v3.vB;
				pVBNI[(dwFaces * 3) + 2].fFace  = (FLOAT) dwFaces;
				pVBNI[(dwFaces * 3) + 2].fBary1 = 0.0;
				pVBNI[(dwFaces * 3) + 2].fBary2 = 1.0;
			}

			m_pVBNI->Unlock( );

			pVBClone->Unlock( );
			pIBClone->Unlock( );
			pVBClone->Release( );
			pIBClone->Release( );

			SAFE_RELEASE( pCloneMesh );

			m_bObjSetUp = true;
		}
	}
	
	return hrResult;
} // CApp::SetUpObject


////////////////////////////////////////////////////////////////////////////////
/// \brief Helper function for drawing on a texture.
///
/// This function is called by CApp::Render when the user is drawing to a
/// texture.
///
/// \return None.
////////////////////////////////////////////////////////////////////////////////
void CApp::DrawAt( BYTE *pTexBmp, D3DLOCKED_RECT &rcLockedRect, int iX, int iY, 
			       int iWidth, int iHeight, BYTE bR, BYTE bG, BYTE bB, BYTE bA )
{
	float fInterp = (float) bA / 255.0f;
	float f1MInterp = 1.0f - fInterp;

	if( iX >= 0 && iX <= iWidth &&
		iY >= 0 && iY <= iHeight )
	{
		int iOffset = (rcLockedRect.Pitch * iY) + (iX * 4);
		pTexBmp[iOffset  ] = (BYTE)((float)bB*fInterp + 
							 (float)pTexBmp[iOffset  ]*f1MInterp); // B
		pTexBmp[iOffset+1] = (BYTE)((float)bG*fInterp + 
							 (float)pTexBmp[iOffset+1]*f1MInterp); // G
		pTexBmp[iOffset+2] = (BYTE)((float)bR*fInterp + 
							 (float)pTexBmp[iOffset+2]*f1MInterp); // R
		pTexBmp[iOffset+3] = (BYTE)((float)bA*fInterp + 
							 (float)pTexBmp[iOffset+3]*f1MInterp); // A
	}
} // CApp::DrawAt


////////////////////////////////////////////////////////////////////////////////
/// \brief Render procedure.
///
/// Called once per frame, the call is the entry point for 3D rendering. 
/// This function sets up render states, clears the viewport, and renders 
/// the scene.
///
/// \return \c S_OK on success.
////////////////////////////////////////////////////////////////////////////////
HRESULT CApp::Render( )
{
	HRESULT hr;

	TCHAR strBuffer[512];
	static float fDepthData;
	static float fPixelA = 0, fPixelR = 0, fPixelG = 0, fPixelB = 0;
	static float fBaryF, fBaryG;
	static float fPickTexU, fPickTexV;
	static float fPickFace;

	// Center and scale object
	SetUpObject( );

	// Move world
	MyFrameMove( );

	fPixelA = fPixelR = fPixelG = fPixelB = 0;

	// Get current matrices
	m_pd3dDevice->GetTransform( D3DTS_WORLD,      &m_mW );
	m_pd3dDevice->GetTransform( D3DTS_VIEW,       &m_mV );
	m_pd3dDevice->GetTransform( D3DTS_PROJECTION, &m_mP );
	m_mWV  = m_mW  * m_mV;
	m_mVP  = m_mV  * m_mP;
	m_mWVP = m_mWV * m_mP;

    // Set up the cursor
    POINT ptCursor;
    GetCursorPos( &ptCursor );
    ScreenToClient( m_hWnd, &ptCursor );
    m_pd3dDevice->SetCursorPosition( ptCursor.x, ptCursor.y, 0L );

    UINT cPasses;
	D3DVIEWPORT9 vp;

	D3DXVECTOR3 vUnProj1;
	D3DXVECTOR4 vUnProj2;
	FLOAT fNormalX, fNormalY, fNormalZ;

	// Hardware picking?
	// Set up viewport and view-proj matrices for picking
	SetSnapMatrix( ptCursor.x, ptCursor.y );

	// Set effect's variables
	UpdateShaderMatrixHandles( );

	// Set render target
	hr = m_pd3dDevice->SetRenderTarget( 0, m_pRTSurface );
	if( FAILED( hr ) )
		return hr;

	// Clear the viewport
	m_pd3dDevice->Clear( 0L, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,
						 0x00000000, 1.0f, 0L );

	// Start picking rendering
	if( SUCCEEDED( m_pd3dDevice->BeginScene( ) ) )
	{
		m_pd3dDevice->SetVertexDeclaration( m_pVertexDeclInteract );
		m_pd3dDevice->SetStreamSource( 0, m_pVBNI, 0, 
									   sizeof(D3DVERTEX_INTERACT) );

		hr = m_pEffect->SetTechnique( "SnapRender" );
		hr = m_pEffect->Begin(&cPasses, 0);
		if( FAILED( hr ) ) return hr;

		for( UINT iPass = 0; iPass < cPasses; iPass++ )
		{
			hr = m_pEffect->BeginPass( iPass );
			if( FAILED( hr ) ) return hr;

			// Draw the non-indexed version of the mesh
			m_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLELIST, 0, m_dwNumFaces );

			m_pEffect->EndPass( );
		}
		m_pEffect->End();
		m_pd3dDevice->EndScene();
	}

	// Read back buffer
	if( ptCursor.x >= 0 && 
		ptCursor.y >= 0 &&
		(UINT) ptCursor.x < m_d3dsdBackBuffer.Width && 
		(UINT) ptCursor.y < m_d3dsdBackBuffer.Height )
	{
		// Copy render target to system memory
		hr = m_pd3dDevice->GetRenderTargetData( m_pRTSurface, m_pRTReadSurf );
		if( FAILED( hr ) )
			return hr;

		D3DLOCKED_RECT d3dLockedRect;
		RECT rcRect;
		rcRect.left   = 0;
		rcRect.top    = 0;
		rcRect.right  = 1;
		rcRect.bottom = 1;

		hr = m_pRTReadSurf->LockRect( &d3dLockedRect, &rcRect, 
									  D3DLOCK_READONLY );

		if( SUCCEEDED( hr ) )
		{
			float *pPixels = (float*) d3dLockedRect.pBits;
			fPixelA = pPixels[3];
			fPixelB = pPixels[2];
			fPixelG = pPixels[1];
			fPixelR = pPixels[0];

			// Depth data
			fDepthData = pPixels[3];

			// Face number
			fPickFace = pPixels[1];

			// Decode image-based normals
			fNormalX = ((( (DWORD)pPixels[0]&0x000000FF)     /255.0f)-0.5f)*2.0f;
			fNormalY = (((((DWORD)pPixels[0]&0x0000FF00)>>8 )/255.0f)-0.5f)*2.0f;
			fNormalZ = (((((DWORD)pPixels[0]&0x00FF0000)>>16)/255.0f)-0.5f)*2.0f;

			// Decode barycentric coordinates
			fBaryF = ( (DWORD)pPixels[2]&0x0000FFFF)     /65535.0f;
			fBaryG = (((DWORD)pPixels[2]&0xFFFF0000)>>16)/65535.0f;

			m_pRTReadSurf->UnlockRect( );

			// Fill special vertex buffer with the picked triangle
			D3DVERTEX_INTERACT* pVertices;
			m_pVBNI->Lock( 0, 0, (void**)&pVertices, 0 );

			D3DVERTEX_SIMPLE* v;
			D3DVERTEX_SIMPLE* vThisTri;

			m_pVBPicked->Lock( 0, 0, (void**)&v, 0 );

			vThisTri = &v[0];

			// Get vertices hit
			vThisTri[0].vP = pVertices[(3*(DWORD)fPickFace)].vP;
			vThisTri[0].vN = pVertices[(3*(DWORD)fPickFace)].vN;
			vThisTri[0].fU = pVertices[(3*(DWORD)fPickFace)].fU;
			vThisTri[0].fV = pVertices[(3*(DWORD)fPickFace)].fV;

			vThisTri[1].vP = pVertices[(3*(DWORD)fPickFace)+1].vP;
			vThisTri[1].vN = pVertices[(3*(DWORD)fPickFace)+1].vN;
			vThisTri[1].fU = pVertices[(3*(DWORD)fPickFace)+1].fU;
			vThisTri[1].fV = pVertices[(3*(DWORD)fPickFace)+1].fV;

			vThisTri[2].vP = pVertices[(3*(DWORD)fPickFace)+2].vP;
			vThisTri[2].vN = pVertices[(3*(DWORD)fPickFace)+2].vN;
			vThisTri[2].fU = pVertices[(3*(DWORD)fPickFace)+2].fU;
			vThisTri[2].fV = pVertices[(3*(DWORD)fPickFace)+2].fV;

			/// Calculate texture coordinates at the pick point
			FLOAT dntu1 = vThisTri[1].fU - vThisTri[0].fU;
			FLOAT dntu2 = vThisTri[2].fU - vThisTri[0].fU;
			FLOAT dntv1 = vThisTri[1].fV - vThisTri[0].fV;
			FLOAT dntv2 = vThisTri[2].fV - vThisTri[0].fV;
			fPickTexU   = vThisTri[0].fU + fBaryF * dntu1 + fBaryG * dntu2;
			fPickTexV   = vThisTri[0].fV + fBaryF * dntv1 + fBaryG * dntv2;
			///

			m_pVBPicked->Unlock();

			m_pVBNI->Unlock( );
		}
	}

	if( fPixelA || fPixelR || fPixelG || fPixelB )
	{
		// Restore viewport size
		vp.X      = 0;
		vp.Y      = 0;
		vp.Width  = m_d3dsdBackBuffer.Width;
		vp.Height = m_d3dsdBackBuffer.Height;
		vp.MinZ   = 0.0f;
		vp.MaxZ   = 1.0f;
		hr = m_pd3dDevice->SetViewport( &vp );
		if( FAILED( hr ) )
			return hr;

		// Get current settings
		D3DXMATRIX mCurProj;
		m_pd3dDevice->GetTransform( D3DTS_PROJECTION, &mCurProj );
		D3DXVECTOR3 vMouse = D3DXVECTOR3( (float) ptCursor.x, 
										  (float) ptCursor.y, 
										  fDepthData );
		D3DXVec3Unproject( &vUnProj1, &vMouse, &vp, &mCurProj, &m_mV, &m_mW );

		m_vTriadNormal = D3DXVECTOR3( fNormalX, fNormalY, fNormalZ );

		D3DXMATRIX mWorldInv = m_mW;
		D3DXMatrixInverse( &mWorldInv, NULL, &m_mW );
		D3DXVec3TransformNormal( &m_vTriadNormal, &m_vTriadNormal, &mWorldInv );
		D3DXVec3Normalize( &m_vTriadNormal, &m_vTriadNormal );

		m_vTriadPos = vUnProj1;
	}

	// Back to original projection
	m_pd3dDevice->SetTransform( D3DTS_PROJECTION, &m_mP );

	// Back to original render target
	hr = m_pd3dDevice->SetRenderTarget( 0, m_pBackBufferSurf );
	if( FAILED( hr ) )
		return hr;

	// Draw on decal texture
	if( fPixelA || fPixelR || fPixelG || fPixelB )
	{
		if( GetAsyncKeyState( VK_F9 ) )
		{
			D3DSURFACE_DESC d3dDesc;
			hr = m_pTex->GetLevelDesc( 0, &d3dDesc );
			if( FAILED( hr ) ) return hr;

			// Calculate pixel coordinates inside the texture
			int iX = (int)(fPickTexU * d3dDesc.Width) % d3dDesc.Width;
			int iY = (int)(fPickTexV * d3dDesc.Height) % d3dDesc.Height;

			{
				D3DLOCKED_RECT rcLockedRect;
				hr = m_pTex->LockRect( 0, &rcLockedRect, NULL, 0 );
				if( FAILED( hr ) ) return hr;

				BYTE bR = (BYTE) ((float)((m_vTriadNormal.x+0.9f)/2.0f)*255.0f);
				BYTE bG = (BYTE) ((float)((m_vTriadNormal.y+0.9f)/2.0f)*255.0f);
				BYTE bB = (BYTE) ((float)((m_vTriadNormal.z+0.9f)/2.0f)*255.0f);

				BYTE* pTexBmp = (BYTE*)rcLockedRect.pBits;
				DrawAt( pTexBmp, rcLockedRect, iX  , iY  , d3dDesc.Width, 
						d3dDesc.Height, bR, bG, bB, 255 );
				DrawAt( pTexBmp, rcLockedRect, iX  , iY-1, d3dDesc.Width, 
						d3dDesc.Height, bR, bG, bB, 50 );
				DrawAt( pTexBmp, rcLockedRect, iX  , iY+1, d3dDesc.Width, 
						d3dDesc.Height, bR, bG, bB, 50 );

				DrawAt( pTexBmp, rcLockedRect, iX-1, iY  , d3dDesc.Width, 
						d3dDesc.Height, bR, bG, bB, 50 );
				DrawAt( pTexBmp, rcLockedRect, iX-1, iY-1, d3dDesc.Width, 
						d3dDesc.Height, bR, bG, bB, 30 );
				DrawAt( pTexBmp, rcLockedRect, iX-1, iY+1, d3dDesc.Width, 
						d3dDesc.Height, bR, bG, bB, 30 );

				DrawAt( pTexBmp, rcLockedRect, iX+1, iY  , d3dDesc.Width, 
						d3dDesc.Height, bR, bG, bB, 50 );
				DrawAt( pTexBmp, rcLockedRect, iX+1, iY-1, d3dDesc.Width, 
						d3dDesc.Height, bR, bG, bB, 30 );
				DrawAt( pTexBmp, rcLockedRect, iX+1, iY+1, d3dDesc.Width, 
						d3dDesc.Height, bR, bG, bB, 30 );

				hr = m_pTex->UnlockRect( 0 );
				if( FAILED( hr ) ) return hr;
			}
		}
	}

	//
	// Actual rendering
	//
	m_pd3dDevice->Clear( 0L, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER,
 						 0x00000000, 1.0f, 0L );

	// Set effect's variables
	UpdateShaderMatrixHandles( );

	// Start rendering procedure
	if( SUCCEEDED( m_pd3dDevice->BeginScene() ) )
	{
		D3DXVECTOR4 vDiffuse = D3DXVECTOR4( 0.7f, 0.7f, 0.7f, 1.0f );
		m_pEffect->SetVector( m_hDiffuse, &vDiffuse );

		D3DXVECTOR4 vAmbient = D3DXVECTOR4( 0.15f, 0.15f, 0.15f, 1.0f );
		m_pEffect->SetVector( m_hAmbient, &vAmbient );

		if( m_bShowNormalData )
			hr = m_pEffect->SetTechnique( "NormalRender" );
		else if( m_bShowDepthData )
			hr = m_pEffect->SetTechnique( "DepthRender" );
		else if( m_bShowFaceID )
			hr = m_pEffect->SetTechnique( "FaceRender" );
		else if( m_bShowBaryData )
			hr = m_pEffect->SetTechnique( "BaryRender" );
		else
			hr = m_pEffect->SetTechnique( "ActualRender" );

		// When showing the face ID and barycentric data, we need
		// to use the non-indexed geometry
		if( m_bShowFaceID || m_bShowBaryData )
		{
			m_pd3dDevice->SetVertexDeclaration( m_pVertexDeclInteract );
			m_pd3dDevice->SetStreamSource( 0, m_pVBNI, 0, 
										   sizeof(D3DVERTEX_INTERACT ) );
		}
		else
		{
			// Use indexed geometry by default
			m_pd3dDevice->SetVertexDeclaration( m_pVertexDeclActual );
			m_pd3dDevice->SetStreamSource( 0, m_pVB, 0, 
										   sizeof(D3DVERTEX_ACTUAL) );
			m_pd3dDevice->SetIndices( m_pIB );
		}

		hr = m_pEffect->Begin(&cPasses, 0);
		if( FAILED( hr ) ) return hr;

		for( UINT iPass = 0; iPass < cPasses; iPass++ )
		{
			hr = m_pEffect->BeginPass( iPass );
			if( FAILED( hr ) ) return hr;

			if( m_bShowFaceID || m_bShowBaryData )
			{
				// Draw the non-indexed version of the mesh
				m_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLELIST, 0, 
											 m_dwNumFaces );
			}
			else
			{
				// Draw the indexed version of the mesh
				m_pd3dDevice->DrawIndexedPrimitive( D3DPT_TRIANGLELIST, 0, 0, 
													m_dwNumVerts, 0, 
													m_dwNumFaces );
			}

			m_pEffect->EndPass( );
		}
		m_pEffect->End();
		m_pd3dDevice->EndScene();
	}

	// If a triangle is picked, draw it
	if( (fPixelA || fPixelR || fPixelG || fPixelB) &&
		m_bShowSnapTriangle )
	{
		m_pd3dDevice->SetTransform( D3DTS_WORLD, &m_mW );

		// Set render mode to unlit, solid triangles
        m_pd3dDevice->SetRenderState( D3DRS_FILLMODE, D3DFILL_SOLID );
        m_pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE );

		// Draw the picked triangle
		m_pd3dDevice->SetVertexDeclaration( m_pVertexDeclSimple );
		m_pd3dDevice->SetStreamSource( 0, m_pVBPicked, 0, 
									   sizeof(D3DVERTEX_SIMPLE) );

		// Start rendering procedure
		if( SUCCEEDED( m_pd3dDevice->BeginScene() ) )
		{
			D3DXVECTOR4 vDiffuse = D3DXVECTOR4( 0.0, 0.0, 0.0, 1.0 );
			m_pEffect->SetVector( m_hDiffuse, &vDiffuse );

			D3DXVECTOR4 vAmbient = D3DXVECTOR4( 0.5f, 0.5f, 0.5f, 1.0f );
			m_pEffect->SetVector( m_hAmbient, &vAmbient );

			hr = m_pEffect->SetTechnique( "ActualRender" );
			hr = m_pEffect->Begin(&cPasses, 0);
			if( FAILED( hr ) ) return hr;

			for( UINT iPass = 0; iPass < cPasses; iPass++ )
			{
				hr = m_pEffect->BeginPass( iPass );
				if( FAILED( hr ) ) return hr;

				m_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLELIST, 0, 1 );

				m_pEffect->EndPass( );
			}
			m_pEffect->End();
			m_pd3dDevice->EndScene();
		}

		m_pd3dDevice->SetVertexDeclaration( m_pVertexDeclInteract );
		m_pd3dDevice->SetStreamSource( 0, m_pVBNI, 0, 
									   sizeof(D3DVERTEX_INTERACT) );
	}

	if( m_bDrawTriad )
	{
		if( fPixelA || fPixelR || fPixelG || fPixelB )
		{
			// Disable mouse cursor
			if( g_bShowCursor )
			{
				g_bShowCursor = false;
				ShowCursor( false );
			}

			// Draw triad
			m_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_CW );

			// Set diffuse color
			D3DXVECTOR4 vDiffuse = D3DXVECTOR4( 1.0, 1.0, 1.0, 1.0 );
			m_pEffect->SetVector( m_hDiffuse, &vDiffuse );

			// Set ambient color
			D3DXVECTOR4 vAmbient = D3DXVECTOR4( 0.05f, 0.05f, 0.05f, 1.0f );
			m_pEffect->SetVector( m_hAmbient, &vAmbient );

			D3DXMATRIX matWorld, matRes, matTemp;

			// Draw center of the triad
			D3DXMatrixTranslation( &matWorld, 
				m_vTriadPos.x, m_vTriadPos.y, m_vTriadPos.z );
			D3DXMatrixMultiply( &matWorld, &matWorld, &m_mW );

			// Align triad
			D3DXVECTOR3 vDir = m_vTriadNormal;

			D3DXVECTOR3 vXAxis, vYAxis, vZAxis;
			D3DXVECTOR3 vUp = D3DXVECTOR3( 1.0, 0.0f, 0.0f );
			vYAxis = vDir;

			if( vDir == vUp )
				vZAxis = D3DXVECTOR3( 0.0f, 0.0f, 1.0 );
			else
				D3DXVec3Cross( &vZAxis, &vDir, &vUp );

			D3DXVec3Normalize( &vZAxis, &vZAxis );
			D3DXVec3Cross    ( &vXAxis, &vZAxis, &vYAxis );
			D3DXVec3Normalize( &vXAxis, &vXAxis );

			D3DXMatrixIdentity( &matTemp );
			matTemp._11 = vXAxis.x;
			matTemp._12 = vXAxis.y;
			matTemp._13 = vXAxis.z;
			matTemp._21 = vYAxis.x;
			matTemp._22 = vYAxis.y;
			matTemp._23 = vYAxis.z;
			matTemp._31 = vZAxis.x;
			matTemp._32 = vZAxis.y;
			matTemp._33 = vZAxis.z;

			D3DXMatrixMultiply( &matWorld, &matTemp, &matWorld );

			m_pd3dDevice->SetTransform( D3DTS_WORLD, &matWorld );
			UpdateShaderMatrixHandles( );

			if( SUCCEEDED( m_pd3dDevice->BeginScene() ) )
			{
				hr = m_pEffect->SetTechnique( "CursorRender" );
				hr = m_pEffect->Begin(&cPasses, 0);
				if( FAILED( hr ) ) return hr;

				for( UINT iPass = 0; iPass < cPasses; iPass++ )
				{
					hr = m_pEffect->BeginPass( iPass );
					if( FAILED( hr ) ) return hr;

					m_pTriadCenter->DrawSubset( 0 );

					m_pEffect->EndPass( );
				}
				m_pEffect->End();
				m_pd3dDevice->EndScene();
			}

			// Axis Z
			D3DXMatrixTranslation( &matTemp, 0.0f, 0.0f, -0.15f );
			D3DXMatrixMultiply( &matRes, &matTemp, &matWorld );
			m_pd3dDevice->SetTransform( D3DTS_WORLD, &matRes );
			UpdateShaderMatrixHandles( );

			vDiffuse = D3DXVECTOR4( 0.0, 0.0, 1.0, 1.0 );
			m_pEffect->SetVector( m_hDiffuse, &vDiffuse );

			vAmbient = D3DXVECTOR4( 0.0, 0.0, 0.25, 1.0 );
			m_pEffect->SetVector( m_hAmbient, &vAmbient );

			if( SUCCEEDED( m_pd3dDevice->BeginScene() ) )
			{
				hr = m_pEffect->SetTechnique( "CursorRender" );
				hr = m_pEffect->Begin(&cPasses, 0);
				if( FAILED( hr ) ) return hr;

				for( UINT iPass = 0; iPass < cPasses; iPass++ )
				{
					hr = m_pEffect->BeginPass( iPass );
					if( FAILED( hr ) ) return hr;

					m_pTriadAxis->DrawSubset( 0 );

					m_pEffect->EndPass( );
				}
				m_pEffect->End();
				m_pd3dDevice->EndScene();
			}

			// Axis X
			D3DXMatrixRotationY( &matTemp, -D3DX_PI / 2.0f );
			D3DXMatrixMultiply( &matRes, &matTemp, &matWorld );
			D3DXMatrixTranslation( &matTemp, 0.0f, 0.0f, -0.15f );
			D3DXMatrixMultiply( &matRes, &matTemp, &matRes );
			m_pd3dDevice->SetTransform( D3DTS_WORLD, &matRes );
			UpdateShaderMatrixHandles( );

			vDiffuse = D3DXVECTOR4( 1.0, 0.0, 0.0, 1.0 );
			m_pEffect->SetVector( m_hDiffuse, &vDiffuse );

			vAmbient = D3DXVECTOR4( 0.25, 0.0, 0.0, 1.0 );
			m_pEffect->SetVector( m_hAmbient, &vAmbient );

			if( SUCCEEDED( m_pd3dDevice->BeginScene() ) )
			{
				hr = m_pEffect->SetTechnique( "CursorRender" );
				hr = m_pEffect->Begin(&cPasses, 0);
				if( FAILED( hr ) ) return hr;

				for( UINT iPass = 0; iPass < cPasses; iPass++ )
				{
					hr = m_pEffect->BeginPass( iPass );
					if( FAILED( hr ) ) return hr;

					m_pTriadAxis->DrawSubset( 0 );

					m_pEffect->EndPass( );
				}
				m_pEffect->End();
				m_pd3dDevice->EndScene();
			}

			// Axis Y
			D3DXMatrixRotationX( &matTemp, D3DX_PI / 2.0f );
			D3DXMatrixMultiply( &matRes, &matTemp, &matWorld );
			D3DXMatrixTranslation( &matTemp, 0.0f, 0.0f, -0.15f );
			D3DXMatrixMultiply( &matRes, &matTemp, &matRes );
			m_pd3dDevice->SetTransform( D3DTS_WORLD, &matRes );

			UpdateShaderMatrixHandles( );

			vDiffuse = D3DXVECTOR4( 0.0, 1.0, 0.0, 1.0 );
			m_pEffect->SetVector( m_hDiffuse, &vDiffuse );

			vAmbient = D3DXVECTOR4( 0.0, 0.25, 0.0, 1.0 );
			m_pEffect->SetVector( m_hAmbient, &vAmbient );

			if( SUCCEEDED( m_pd3dDevice->BeginScene() ) )
			{
				hr = m_pEffect->SetTechnique( "CursorRender" );
				hr = m_pEffect->Begin(&cPasses, 0);
				if( FAILED( hr ) ) return hr;

				for( UINT iPass = 0; iPass < cPasses; iPass++ )
				{
					hr = m_pEffect->BeginPass( iPass );
					if( FAILED( hr ) ) return hr;

					m_pTriadAxis->DrawSubset( 0 );

					m_pEffect->EndPass( );
				}
				m_pEffect->End();
				m_pd3dDevice->EndScene();
			}
			m_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_CCW );
		}
		else
		{
			// Re-enable mouse cursor
			if( !g_bShowCursor )
			{
				g_bShowCursor = true;
				ShowCursor( true );
			}
		}
	}

	// Output statistics
	m_pFont->DrawText( 10, 10, D3DCOLOR_ARGB(255,255,255,255), 
					   m_strFrameStats );

	if( m_bShowNormalData )
		m_pFont->DrawText( 160, 10, D3DCOLOR_ARGB(255,255,255,255), 
						   _T("Normal map rendering") );
	if( m_bShowDepthData )
		m_pFont->DrawText( 160, 10, D3DCOLOR_ARGB(255,255,255,255), 
						   _T("Depth map rendering") );
	if( m_bShowFaceID )
		m_pFont->DrawText( 160, 10, D3DCOLOR_ARGB(255,255,255,255), 
						   _T("Labeling of faces") );
	if( m_bShowBaryData )
		m_pFont->DrawText( 160, 10, D3DCOLOR_ARGB(255,255,255,255),
						   _T("Barycentric coordinates") );

	if( !m_bShowNormalData && !m_bShowDepthData && 
		!m_bShowFaceID     && !m_bShowBaryData )
		m_pFont->DrawText( 160, 10, D3DCOLOR_ARGB(255,255,255,255), 
						   _T("Actual rendering") );

	// Output help
	FLOAT yText = 50.0f;
	if( m_bShowHelp )
	{
		m_pFontSmall->DrawText( 10, yText, D3DCOLOR_ARGB(255,192,192,192), 
								_T("F1: Toggle Help") );
		yText += 15.0f;
		m_pFontSmall->DrawText( 10, yText, D3DCOLOR_ARGB(255,192,192,192), 
								_T("F2: Toggle normal map rendering") );
		yText += 15.0f;
		m_pFontSmall->DrawText( 10, yText, D3DCOLOR_ARGB(255,192,192,192), 
								_T("F3: Toggle depth map rendering") );
		yText += 15.0f;
		m_pFontSmall->DrawText( 10, yText, D3DCOLOR_ARGB(255,192,192,192), 
								_T("F4: Show/hide faces' IDs") );
		yText += 15.0f;
		m_pFontSmall->DrawText( 10, yText, D3DCOLOR_ARGB(255,192,192,192), 
								_T("F5: Show/hide barycentric weights") );
		yText += 15.0f;
		m_pFontSmall->DrawText( 10, yText, D3DCOLOR_ARGB(255,192,192,192), 
								_T("F6: Show/hide triangle under the cursor") );
		yText += 15.0f;
		m_pFontSmall->DrawText( 10, yText, D3DCOLOR_ARGB(255,192,192,192), 
								_T("F9: Enable 3D painting (when hold down)") );
		yText += 15.0f;
		m_pFontSmall->DrawText( 10, yText, D3DCOLOR_ARGB(255,192,192,192), 
								_T("Up/Down: Zoom in/out") );
		yText += 15.0f;
		m_pFontSmall->DrawText( 10, yText, D3DCOLOR_ARGB(255,192,192,192), 
								_T("Enter: Toggle animation") );
		yText += 15.0f;
		m_pFontSmall->DrawText( 10, yText, D3DCOLOR_ARGB(255,192,192,192), 
								_T("Space: Single-step frame movement") );
		yText += 15.0f;
	}
	else
	{
		m_pFontSmall->DrawText( 10, yText, D3DCOLOR_ARGB(255,192,192,192), 
								_T("Press F1 for Help") );
		yText += 15.0f;
	}


	// Output info
	yText = 10.0f;

	if( fPixelA || fPixelR || fPixelG || fPixelB )
	{

		_stprintf( strBuffer,
			_T("Face=%d   Bary=%3.02f, %3.02f   Tex=%3.02f, %3.02f\n")
			_T("Pos=%3.02f, %3.02f, %3.02f   Normal=%3.02f, %3.02f, %3.02f"), 
			(DWORD) fPickFace,
			fBaryF, fBaryG,
			fPickTexU, fPickTexV,
			m_vTriadPos.x, m_vTriadPos.y, m_vTriadPos.z,
			m_vTriadNormal.x, m_vTriadNormal.y, m_vTriadNormal.z );
		m_pFont->DrawText( 10.0f, (FLOAT)(m_d3dsdBackBuffer.Height - 50), 
						   D3DCOLOR_ARGB(255,255,255,0), strBuffer );
	} 

    return S_OK;
} // CApp::Render


////////////////////////////////////////////////////////////////////////////////
/// \brief User-defined FrameMove( ).
///
/// \return None.
////////////////////////////////////////////////////////////////////////////////
void CApp::MyFrameMove( )
{
	// Change z value of the viewer position
	if( GetAsyncKeyState( VK_UP   ) ) m_fZDist += 0.05f;
	if( GetAsyncKeyState( VK_DOWN ) ) m_fZDist -= 0.05f;

	static DWORD tLast = timeGetTime( );
	DWORD tNow = timeGetTime( );

	// Rotate object over time
	D3DXMATRIXA16 mWorld, mRotY, mRotZ, mTrans;
	D3DXMatrixIdentity( &mWorld );
	D3DXMatrixRotationY( &mRotY, m_fTime/20.0f );
	D3DXMatrixRotationZ( &mRotZ, m_fTime/20.0f );

	// Apply transformations
	D3DXMatrixMultiply( &mWorld, &mWorld, &mRotY );
	D3DXMatrixMultiply( &mWorld, &mWorld, &mRotZ );

	// Translate in Z
	D3DXMatrixTranslation( &mTrans, 0, 0, -m_fZDist - 5 );
	D3DXMatrixMultiply( &mWorld, &mWorld, &mTrans );
    m_pd3dDevice->SetTransform( D3DTS_WORLD, &mWorld );
} // CApp::MyFrameMove


////////////////////////////////////////////////////////////////////////////////
/// \brief Updates the parameters of the snapping effect.
///
/// Updates the shader parameters according to the current world, view and
/// projection matrices.
///
/// \return None.
////////////////////////////////////////////////////////////////////////////////
void CApp::UpdateShaderMatrixHandles( )
{
	D3DXMATRIX mWorld, mView, mProj;

	// Get current world, view, proj matrices
	m_pd3dDevice->GetTransform( D3DTS_WORLD,      &mWorld );
	m_pd3dDevice->GetTransform( D3DTS_PROJECTION, &mProj  );
	m_pd3dDevice->GetTransform( D3DTS_VIEW,       &mView  );

	D3DXMATRIX mWorldView     = mWorld * mView;
	D3DXMATRIX mViewProj      = mView * mProj;
	D3DXMATRIX mWorldViewProj = mWorldView * mProj;

	// Set them to the current shader
	m_pEffect->SetMatrix( m_hWorldViewProjection, &mWorldViewProj );
	m_pEffect->SetMatrix( m_hWorld, &mWorld );
	m_pEffect->SetMatrix( m_hView, &mView );

	m_pEffect->SetFloat( m_hDeform1, m_fDeform1 );
	m_pEffect->SetFloat( m_hDeform2, m_fDeform2 );
} // CApp::UpdateShaderMatrixHandles


////////////////////////////////////////////////////////////////////////////////
/// \brief Initializes the D3D device-dependent objects.
///
/// Paired with CApp::DeleteDeviceObjects( ).
///
/// \return None.
////////////////////////////////////////////////////////////////////////////////
HRESULT CApp::InitDeviceObjects( )
{
	HRESULT hr;

    // Restore the fonts
    m_pFont->InitDeviceObjects( m_pd3dDevice );
    m_pFontSmall->InitDeviceObjects( m_pd3dDevice );

	// Create vertex declarations
	hr = m_pd3dDevice->CreateVertexDeclaration( g_VDeclInteract, 
												&m_pVertexDeclInteract );
	if( FAILED( hr ) ) return hr;

	hr = m_pd3dDevice->CreateVertexDeclaration( g_VDeclActual, 
												&m_pVertexDeclActual );
	if( FAILED( hr ) ) return hr;

	hr = m_pd3dDevice->CreateVertexDeclaration( g_VDeclSimple, 
												&m_pVertexDeclSimple );
	if( FAILED( hr ) ) return hr;

	// Create texture and bump texture
	hr = D3DXCreateTextureFromFile( m_pd3dDevice, _T("plain.bmp"), &m_pTex );
	if( FAILED( hr ) ) return hr;

	hr = D3DXCreateTextureFromFile( m_pd3dDevice, _T("bump.dds"), &m_pBumpTex );
	if( FAILED( hr ) ) return hr;

	// Create meshes for showing the picking point
	if( FAILED( D3DXCreateSphere( m_pd3dDevice, 0.05f, 10, 10, 
								  &m_pTriadCenter, NULL ) ) )
        return E_FAIL;

	if( FAILED( D3DXCreateCylinder( m_pd3dDevice, 0.03f, 0.02f, 0.25f, 10, 1, 
								    &m_pTriadAxis, NULL ) ) )
        return E_FAIL;

	// Load mesh to be picked
	m_bObjSetUp = false;
    if( FAILED( m_pObject->Create( m_pd3dDevice, _T("sphere.x") ) ) )
        return D3DAPPERR_MEDIANOTFOUND;
	m_pObject->UseMeshMaterials( false );
	hr = m_pObject->SetVertexDecl( m_pd3dDevice, g_VDeclSimple );
	if( FAILED( hr ) ) return hr;

    // Create the vertex buffer for picked triangles
    if( FAILED( m_pd3dDevice->CreateVertexBuffer( 
		3 * sizeof(D3DVERTEX_SIMPLE),
		D3DUSAGE_WRITEONLY, D3DVERTEX_SIMPLE::FVF,
        D3DPOOL_MANAGED, &m_pVBPicked, NULL ) ) )
    {
        return E_FAIL;
    }

    // Define DEBUG_VS and/or DEBUG_PS to debug vertex and/or pixel shaders 
	// with the shader debugger. Debugging vertex shaders requires either REF 
	// or software vertex processing, and debugging pixel shaders requires REF.  
	// The D3DXSHADER_FORCE_*_SOFTWARE_NOOPT flag improves the debug experience 
	// in the shader debugger.  It enables source level debugging, prevents 
	// instruction reordering, prevents dead code elimination, and forces the 
	// compiler to compile against the next higher available software target, 
	// which ensures that the unoptimized shaders do not exceed the shader model 
	// limitations.  Setting these flags will cause slower rendering since the 
	// shaders will be unoptimized and forced into software.  See the DirectX 
	// documentation for more information about using the shader debugger.
    DWORD dwShaderFlags = 0;
    #ifdef DEBUG_VS
        dwShaderFlags |= D3DXSHADER_FORCE_VS_SOFTWARE_NOOPT;
    #endif
    #ifdef DEBUG_PS
        dwShaderFlags |= D3DXSHADER_FORCE_PS_SOFTWARE_NOOPT;
    #endif

    // Read the D3DX effect file
    TCHAR str[MAX_PATH];
    hr = DXUtil_FindMediaFileCb( str, sizeof(str), TEXT("snapping.fx") );
    if( FAILED(hr) )
		return E_FAIL;

    hr = D3DXCreateEffectFromFile( m_pd3dDevice, str, NULL, NULL, dwShaderFlags, 
								   NULL, &m_pEffect, NULL );

    // If this fails, there should be debug output as to 
    // they the .fx file failed to compile
    if( FAILED( hr ) )
		return E_FAIL;

    return S_OK;
} // CApp::InitDeviceObjects


////////////////////////////////////////////////////////////////////////////////
/// \brief Initializes and restores the D3D device objects.
///
/// Paired with CApp::InvalidateDeviceObjects( ).
///
/// \return \c S_Ok on success.
////////////////////////////////////////////////////////////////////////////////
HRESULT CApp::RestoreDeviceObjects( )
{
	HRESULT hrResult = S_OK;

	m_bObjSetUp = false;

    m_pFont->RestoreDeviceObjects();
    m_pFontSmall->RestoreDeviceObjects();
    if( m_pEffect )
        m_pEffect->OnResetDevice();

	// Create render target texture
	hrResult = D3DXCreateTexture( m_pd3dDevice,
								  1, 1, 
								  1,
								  D3DUSAGE_RENDERTARGET,
								  RT_FORMAT,
								  D3DPOOL_DEFAULT,
								  &m_pRTTexture );
	if( FAILED( hrResult ) )
		return hrResult;

	// Get surface of the render target
	hrResult = m_pRTTexture->GetSurfaceLevel( 0, &m_pRTSurface );
	if( FAILED( hrResult ) )
		return hrResult;

	// Create off-screen plain surface for reading the render target
	// to system memory
	hrResult = m_pd3dDevice->CreateOffscreenPlainSurface(
		1, 1,
		RT_FORMAT,
		D3DPOOL_SYSTEMMEM,
		&m_pRTReadSurf,
		NULL );
	if( FAILED( hrResult ) )
		return hrResult;

	// Get back buffer surface
	hrResult = m_pd3dDevice->GetBackBuffer( 0, 0, 
		D3DBACKBUFFER_TYPE_MONO, &m_pBackBufferSurf );
	if( FAILED( hrResult ) ) 
		return hrResult;

    // To read or write to D3DX effect variables we can use the string name 
    // instead of using handles, however it improves perf to use handles since
    // then D3DX won't have to spend time doing string compares
    m_hWorld               = m_pEffect->GetParameterByName( NULL, "g_mW" );
	m_hView                = m_pEffect->GetParameterByName( NULL, "g_mV" );
    m_hWorldViewProjection = m_pEffect->GetParameterByName( NULL, "g_mWVP" );
	m_hDiffuse             = m_pEffect->GetParameterByName( NULL, "g_vDiffuse" );
	m_hAmbient             = m_pEffect->GetParameterByName( NULL, "g_vAmbient" );
	m_hDeform1             = m_pEffect->GetParameterByName( NULL, "g_fDeform1" );
	m_hDeform2             = m_pEffect->GetParameterByName( NULL, "g_fDeform2" );

    // Restore device-memory objects for the mesh
    m_pObject->RestoreDeviceObjects( m_pd3dDevice );

    // Set up the textures
    m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
    m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
    m_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1);
    m_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
    m_pd3dDevice->SetSamplerState( 0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR );
    m_pd3dDevice->SetSamplerState( 0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR );
    m_pd3dDevice->SetSamplerState( 0, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR );

    // Set miscellaneous render states
    m_pd3dDevice->SetRenderState( D3DRS_DITHERENABLE,   FALSE );
    m_pd3dDevice->SetRenderState( D3DRS_SPECULARENABLE, FALSE );
    m_pd3dDevice->SetRenderState( D3DRS_ZENABLE,        TRUE );
	m_pd3dDevice->SetRenderState( D3DRS_CULLMODE,       D3DCULL_CCW );

    // Set the projection matrix
    D3DXMATRIXA16 matProj;
    FLOAT fAspect = ((FLOAT)m_d3dsdBackBuffer.Width) / m_d3dsdBackBuffer.Height;
    D3DXMatrixPerspectiveFovLH( &matProj, D3DX_PI/4, fAspect, 
								m_fZNear, m_fZFar );
    m_pd3dDevice->SetTransform( D3DTS_PROJECTION, &matProj );

    return S_OK;
} // CApp::RestoreDeviceObjects( )


////////////////////////////////////////////////////////////////////////////////
/// Invalidates the objects initialized or restored in 
/// CApp::RestoreDeviceObjects( ).
///
/// \return \c S_OK on success.
////////////////////////////////////////////////////////////////////////////////
HRESULT CApp::InvalidateDeviceObjects()
{
	SAFE_RELEASE( m_pRTReadSurf );
	SAFE_RELEASE( m_pRTSurface );
	SAFE_RELEASE( m_pRTTexture );
	SAFE_RELEASE( m_pBackBufferSurf );

    m_pFont->InvalidateDeviceObjects( );
    m_pFontSmall->InvalidateDeviceObjects( );
    if( m_pEffect )
        m_pEffect->OnLostDevice( );
    m_pObject->InvalidateDeviceObjects( );

	SAFE_RELEASE( m_pIB );

    return S_OK;
} // CApp::InvalidateDeviceObjects


////////////////////////////////////////////////////////////////////////////////
/// \brief Deletes device dependent objects.
///
/// Called when the app is exiting, or the device is being changed, this 
/// function deletes any device dependent objects (objects initialized in
/// CApp::InitDeviceObjects( )).
///
/// \return \c S_OK on success.
////////////////////////////////////////////////////////////////////////////////
HRESULT CApp::DeleteDeviceObjects( )
{
	SAFE_RELEASE( m_pVertexDeclInteract );
	SAFE_RELEASE( m_pVertexDeclActual );
	SAFE_RELEASE( m_pVertexDeclSimple );

    m_pFont->DeleteDeviceObjects( );
    m_pFontSmall->DeleteDeviceObjects( );
    m_pObject->Destroy( );

	SAFE_RELEASE( m_pVBPicked  );
	SAFE_RELEASE( m_pVB        );
	SAFE_RELEASE( m_pVBNI      );

	SAFE_RELEASE( m_pEffect      );
	SAFE_RELEASE( m_pTriadCenter );
	SAFE_RELEASE( m_pTriadAxis   );

	SAFE_RELEASE( m_pTex     );
	SAFE_RELEASE( m_pBumpTex );

    return S_OK;
} // CApp::DeleteDeviceObjects


////////////////////////////////////////////////////////////////////////////////
/// \brief Final cleanup.
///
/// Called before the app exits, this function gives the app the chance to 
/// cleanup after itself.
///
/// \return \c S_OK on success.
////////////////////////////////////////////////////////////////////////////////
HRESULT CApp::FinalCleanup( )
{
    SAFE_DELETE( m_pFont      );
    SAFE_DELETE( m_pFontSmall );
    SAFE_DELETE( m_pObject    );

    return S_OK;
} // CApp::FinalCleanup


////////////////////////////////////////////////////////////////////////////////
/// \brief Checks whether the device supports the required capabilities.
///
/// Called during device initialization, this code checks the device for some 
/// minimum set of capabilities.
///
/// \return \c S_OK on success.
////////////////////////////////////////////////////////////////////////////////
HRESULT CApp::ConfirmDevice( D3DCAPS9* pCaps, DWORD dwBehavior,
                             D3DFORMAT adapterFormat, 
							 D3DFORMAT backBufferFormat )
{
    // Debugging vertex shaders requires either REF or software vertex
    // processing and debugging pixel shaders requires REF.  
    #ifdef DEBUG_VS
        if( pCaps->DeviceType != D3DDEVTYPE_REF && 
            (dwBehavior & D3DCREATE_SOFTWARE_VERTEXPROCESSING) == 0 )
            return E_FAIL;
    #endif
    #ifdef DEBUG_PS
        if( pCaps->DeviceType != D3DDEVTYPE_REF )
            return E_FAIL;
    #endif

    if( dwBehavior & D3DCREATE_PUREDEVICE )
        return E_FAIL;

    if( pCaps->VertexShaderVersion < D3DVS_VERSION(1,1) ||
		pCaps->PixelShaderVersion  < D3DPS_VERSION(2,0) )
    {
        if( (dwBehavior & D3DCREATE_SOFTWARE_VERTEXPROCESSING ) == 0 )
        {
            return E_FAIL;
        }
    }

	return S_OK;
} // CApp::ConfirmDevice


////////////////////////////////////////////////////////////////////////////////
/// \brief Processes window messages.
///
/// Overrides the main WndProc, so the sample can do custom message handling 
/// (e.g. processing mouse, keyboard, or menu commands).
///
/// \return The return value of CD3DApplication::MsgProc( ).
////////////////////////////////////////////////////////////////////////////////
LRESULT CApp::MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    switch( msg )
    {
        case WM_LBUTTONDOWN:
            // User pressed left mouse button
            SetCapture( hWnd );
            break;

        case WM_LBUTTONUP:
            // The user released the left mouse button
            ReleaseCapture();
            break;

        case WM_COMMAND:
            if( LOWORD(wParam) == IDM_SHOW_HELP )
                m_bShowHelp = !m_bShowHelp;
			if( LOWORD(wParam) == IDM_TOGGLENORMALMAP )
			{
				m_bShowBaryData   = false;
				m_bShowDepthData  = false;
				m_bShowFaceID     = false;
				m_bShowNormalData = !m_bShowNormalData;
			}
			if( LOWORD(wParam) == IDM_TOGGLEDEPTHMAP )
			{
				m_bShowBaryData   = false;
				m_bShowNormalData = false;
				m_bShowFaceID     = false;
				m_bShowDepthData  = !m_bShowDepthData;
			}
			if( LOWORD(wParam) == IDM_TOGGLEFACEID )
			{
				m_bShowBaryData   = false;
				m_bShowNormalData = false;
				m_bShowDepthData  = false;
				m_bShowFaceID     = !m_bShowFaceID;
			}
			else
			if( LOWORD(wParam) == IDM_TOGGLEBARYMAP )
			{
				m_bShowNormalData = false;
				m_bShowDepthData  = false;
				m_bShowFaceID     = false;
				m_bShowBaryData   = !m_bShowBaryData;
			}
			if( LOWORD(wParam) == IDM_TOGGLESNAPTRIANGLE )
				m_bShowSnapTriangle = !m_bShowSnapTriangle;
            break;
    }

    return CD3DApplication::MsgProc( hWnd, msg, wParam, lParam );
} // CApp::MsgProc