////////////////////////////////////////////////////////////////////////////////
/// \file snapping.fx
///
/// The effect file for snapping to geometry deformed on the GPU.
///
/// (c)2005 Harlen Batagelo, harlen@dca.fee.unicamp.br
////////////////////////////////////////////////////////////////////////////////

#define LSHIFT8  256   ///< Used for left-shifting a value by 8 bits.
#define LSHIFT16 65536 ///< Used for left-shifting a value by 16 bits.

////////////////////////////////////////////////////////////////////////////////
// Global variables
////////////////////////////////////////////////////////////////////////////////
float4x4 g_mW;	 ///< World matrix.
float4x4 g_mV;   ///< View matrix.
float4x4 g_mWVP; ///< World*View*Proj matrix.

float4   g_vDiffuse  = {   0.7,  0.7,   0.7, 1.0 }; ///< Diffuse color.
float4   g_vAmbient  = {   0.0,  0.0,   0.0, 1.0 }; ///< Ambient color.
float3   g_vLightPos = { -10.0, 25.0, -40.0      }; ///< Light position.

float    g_fDeform1  = 10.0f; ///< Deformation level in the Y axis.
float    g_fDeform2  = 20.0f; ///< Deformation level in the Z axis.

////////////////////////////////////////////////////////////////////////////////
// Vertex shader input semantics for the actual rendering.
////////////////////////////////////////////////////////////////////////////////
struct VS_IN
{
	float4 Pos      : POSITION;  ///< Vertex position.
    float3 Normal   : NORMAL;    ///< Vertex normal.
    float2 Tex      : TEXCOORD0; ///< Texture coordinates.
    float3 Tangent  : TANGENT0;  ///< Vertex tangent.
    float3 Binormal : BINORMAL0; ///< Vertex binormal.
};

////////////////////////////////////////////////////////////////////////////////
// Vertex shader output semantics for the actual rendering.
////////////////////////////////////////////////////////////////////////////////
struct VS_OUT
{
    float4 Pos      : POSITION;  ///< Vertex position.
	float2 Tex      : TEXCOORD0; ///< Texture coordinates.
    float3 Normal   : TEXCOORD1; ///< Normal vector in world space.
    float3 EyeVec   : TEXCOORD2; ///< View vector in world space.
    float3 Tangent  : TEXCOORD3; ///< Vertex tangent.
    float3 Binormal : TEXCOORD4; ///< Vertex binormal.
};

////////////////////////////////////////////////////////////////////////////////
// Vertex shader input semantics for rendering the 3D cursor.
////////////////////////////////////////////////////////////////////////////////
struct VS_INCURSOR
{
	float4 vPos    : POSITION; ///< Vertex position.
    float3 vNormal : NORMAL;   ///< Vertex normal.
};

////////////////////////////////////////////////////////////////////////////////
// Vertex shader output semantics for rendering the 3D cursor.
////////////////////////////////////////////////////////////////////////////////
struct VS_OUTCURSOR
{
    float4 Position    : POSITION;  ///< Vertex position.
    float3 Diffuse     : COLOR;     ///< Vertex diffuse color.
};

////////////////////////////////////////////////////////////////////////////////
// Vertex shader input semantics for rendering the snapping data.
////////////////////////////////////////////////////////////////////////////////
struct VS_INSNAP
{ 
	float4 vPos     : POSITION;  ///< Vertex position.
    float3 vNormal  : NORMAL;    ///< Vertex normal.
    float3 vTex     : TEXCOORD0; ///< Bump mapping texture coordinates.
    float2 vBary    : TEXCOORD1; ///< Barycentric weights.
    float3 Tangent  : TANGENT0;  ///< Vertex tangent.
    float3 Binormal : BINORMAL0; ///< Vertex binormal.
};

////////////////////////////////////////////////////////////////////////////////
// Vertex shader output semantics for rendering the snapping data.
////////////////////////////////////////////////////////////////////////////////
struct VS_OUTSNAP
{
    float4 Position    : POSITION;  ///< Vertex position.
    float3 Diffuse     : COLOR;     ///< Diffuse color.
	float2 Tex         : TEXCOORD0; ///< Bump mapping texture coordinates.
    float3 ZWF         : TEXCOORD1; ///< Z in red; W in green; Face ID in blue.
    float2 Bary        : TEXCOORD2; ///< Barycentric weights.
    float3 Tangent     : TEXCOORD3; ///< Tangent.
    float3 Binormal    : TEXCOORD4; ///< Binormal.
};

////////////////////////////////////////////////////////////////////////////////
// Pixel shader output semantics for the actual rendering.
////////////////////////////////////////////////////////////////////////////////
struct PS_OUT
{
    float4 RGBColor : COLOR;  ///< Final pixel color    
};

////////////////////////////////////////////////////////////////////////////////
// Texture samplers
////////////////////////////////////////////////////////////////////////////////

/// Sampler for the decal texture.
texture Tex0;
sampler Sampler0 = sampler_state
{
    Texture   = (Tex0);
    MipFilter = NONE;
    MinFilter = ANISOTROPIC;
    MagFilter = ANISOTROPIC;
};

/// Sampler for the bump map.
texture Tex1;
sampler Sampler1 = sampler_state
{
    Texture   = (Tex1);
    MipFilter = ANISOTROPIC;
    MinFilter = ANISOTROPIC;
    MagFilter = ANISOTROPIC;
};


////////////////////////////////////////////////////////////////////////////////
/// \brief Deform the vertex position and normal by sine functions.
///
/// \param vPos Input vertex position.
/// \param vNormal Input vertex normal.
/// \param vNormalOut Output vertex normal.
/// \param vPosOut Output vertex position.
////////////////////////////////////////////////////////////////////////////////
void ApplyVSDeform( float4 vP, float3 vN, float3 vT, float3 vB,
		            out float4 vPOut, out float3 vNOut, out float3 vTOut,
		            out float3 vBOut ) 
{
	vPOut = vP;
	vNOut = vN;
	vTOut = vT;
	vBOut = vB;

	// Apply vertex deformation
	vPOut = vP + sin(vN.y*g_fDeform1)/25.0 + cos(vN.z*g_fDeform2)/25.0;
	
	// Apply normal deformation
	vNOut.x = vN.x;
	vNOut.y = vN.y + (g_fDeform1/25.0)*cos(g_fDeform1*vN.y);
	vNOut.z = vN.z - (g_fDeform2/25.0)*sin(g_fDeform2*vN.z);

	// Convert to world space	
    vNOut = mul(vNOut, (float3x3)g_mW);
    vTOut = mul(vTOut, (float3x3)g_mW);
    vBOut = mul(vBOut, (float3x3)g_mW);        
}


////////////////////////////////////////////////////////////////////////////////
/// \brief Vertex shader used for encoding snapping data.
////////////////////////////////////////////////////////////////////////////////
VS_OUTSNAP SnapVS( VS_INSNAP In )
{
    VS_OUTSNAP Out;

    // Apply vertex deformation
    float4 vPosOut;
    float3 vNormalOut, vTangentOut, vBinormalOut;
    
    ApplyVSDeform(In.vPos, In.vNormal, In.Tangent, In.Binormal,
				  vPosOut, vNormalOut, vTangentOut, vBinormalOut);

    // Transform the position from object space to 
    // homogeneous projection space
    Out.Position = mul(vPosOut, g_mWVP);

    // Transfer snapping data for pixel shader
    Out.Diffuse  = (vNormalOut/2.0f)+0.5f; // Normal vector
    Out.Tex      = In.vTex.rg;             // Bump mapping tex coords
    Out.ZWF.rg   = Out.Position.zw;        // Depth data (z and w)
    Out.ZWF.b    = In.vTex.b;              // Face ID
    Out.Bary.rg  = In.vBary.rg;            // Barycentric weights
    Out.Tangent  = vTangentOut;            // Tangent
    Out.Binormal = vBinormalOut;           // Binormal
	 
    return Out;
}


////////////////////////////////////////////////////////////////////////////////
/// \brief Pixel shader used for encoding snapping data.
////////////////////////////////////////////////////////////////////////////////
PS_OUT SnapPS( VS_OUTSNAP In ) 
{ 
    PS_OUT Out;

	// Modify normal wrt bump mapping
	float3 fNormal    = (In.Diffuse-0.5f)*2.0f;
    float3 vNormalMap = -((tex2D(Sampler1, In.Tex.rg) * 2.0f) - 1.0);
    fNormal = In.Tangent * vNormalMap.x +
			  In.Binormal * vNormalMap.y +
			  fNormal;
	fNormal = normalize(fNormal);
	
	// Encode normal vector
	fNormal = (fNormal/2.0f)+0.5f;	
    Out.RGBColor.r = (int)(fNormal.r*255)+ 
					 (int)(fNormal.g*255)*LSHIFT8+
					 (int)(fNormal.b*255)*LSHIFT16;

	// Encode face ID
    Out.RGBColor.g = round(In.ZWF.b);
    
    // Encode barycentric coordinates
    Out.RGBColor.b = (int)(In.Bary.r*65535)+
					 (int)(In.Bary.g*65535)*LSHIFT16;
						
	// Encode depth data
    Out.RGBColor.a = In.ZWF.r/In.ZWF.g;
    return Out;
}


////////////////////////////////////////////////////////////////////////////////
/// \brief Pixel shader used for showing the depth data.
////////////////////////////////////////////////////////////////////////////////
PS_OUT DepthPS( VS_OUTSNAP In ) 
{ 
    PS_OUT Out;

	// Encode depth data
    Out.RGBColor.rgb = In.ZWF.r/16;
    Out.RGBColor.a = 1.0;
    return Out;
}


////////////////////////////////////////////////////////////////////////////////
/// \brief Pixel shader used for showing face IDs with false colors.
////////////////////////////////////////////////////////////////////////////////
PS_OUT FacePS( VS_OUTSNAP In ) 
{ 
    PS_OUT Out;

    Out.RGBColor.r = (round(In.ZWF.b) % 256) / 256.0;
    Out.RGBColor.g = (round(In.ZWF.b) % 512) / 512.0;
    Out.RGBColor.b = (round(In.ZWF.b) % 1024) / 1024.0;
    Out.RGBColor.a = 1.0f;
    return Out;
}


////////////////////////////////////////////////////////////////////////////////
/// \brief Pixel shader used for encoding barycentric weights.
////////////////////////////////////////////////////////////////////////////////
PS_OUT BaryPS( VS_OUTSNAP In ) 
{ 
    PS_OUT Out;

    // Encode barycentric coordinates
    Out.RGBColor.r = In.Bary.r;
    Out.RGBColor.g = In.Bary.g;
    Out.RGBColor.b = 0.0;
    Out.RGBColor.a = 1.0f;

    return Out;
}


////////////////////////////////////////////////////////////////////////////////
/// \brief Vertex shader for the actual rendering.
///
/// This shader just computes a diffuse color.
////////////////////////////////////////////////////////////////////////////////
VS_OUT ActualVS( VS_IN In )
{
	VS_OUT Out;

    // Apply vertex deformation
    float4 vPosOut;
    float3 vNormalOut, vTangentOut, vBinormalOut;
    ApplyVSDeform(In.Pos, In.Normal, In.Tangent, In.Binormal,
				  vPosOut, vNormalOut, vTangentOut, vBinormalOut);
//    vNormalOut = In.Normal;
//    vPosOut = In.Pos;

    Out.Normal = vNormalOut;
    float3 PosWorld = mul(vPosOut, g_mW).xyz;
    Out.EyeVec = (PosWorld - g_mV[3].xyz);

    // Transform the position from object space 
    // to homogeneous projection space
    Out.Pos      = mul(vPosOut, g_mWVP);
	Out.Tex      = In.Tex;
	Out.Tangent  = vTangentOut;
	Out.Binormal = vBinormalOut;

    return Out;
}


////////////////////////////////////////////////////////////////////////////////
/// \brief Pixel shader for the actual rendering.
////////////////////////////////////////////////////////////////////////////////
PS_OUT ActualPS( VS_OUT In ) 
{ 
    PS_OUT Out;
  
    float3 vNormalMap = -((tex2D(Sampler1, In.Tex) * 2.0) - 1.0);
	float3 N = In.Tangent * vNormalMap.x + 
			   In.Binormal * vNormalMap.y +
			   In.Normal;
	N = normalize(N);

	// Compute surface color
	float3 L = float3(1.0, -1.0, 1.0); L = normalize(-L);
	float3 E = normalize(In.EyeVec);
   	float3 H = normalize(E + L);
   	
   	float diffContrib = max(0,dot(N,L));
   	float specContrib = pow(max(0,dot(N,H)),128);
   	if( diffContrib <= 0 ) { specContrib = 0; }

    Out.RGBColor = tex2D(Sampler0, In.Tex) * diffContrib + specContrib + g_vAmbient;

    return Out;
}


////////////////////////////////////////////////////////////////////////////////
/// \brief Pixel shader for the rendering the normal vectors.
////////////////////////////////////////////////////////////////////////////////
PS_OUT ActualNormalPS( VS_OUT In ) 
{ 
    PS_OUT Out;
  
    float3 vNormalMap = -((tex2D(Sampler1, In.Tex) * 2.0) - 1.0);
	float3 N = In.Tangent * vNormalMap.x + 
			   In.Binormal * vNormalMap.y +
			   In.Normal;
	N = normalize(N);

    Out.RGBColor.rgb = N;
    Out.RGBColor.a = 1.0;

    return Out;
}


////////////////////////////////////////////////////////////////////////////////
/// \brief Vertex shader for rendering the 3D cursor.
////////////////////////////////////////////////////////////////////////////////
VS_OUTCURSOR CursorVS( VS_INCURSOR In )
{
    VS_OUTCURSOR Out;
   
    // Transform the position from object space 
    // to homogeneous projection space
    Out.Position = mul(In.vPos, g_mWVP);
    
    // Compute a simple directional lighting equation
    Out.Diffuse.rgb = (max(0, dot(In.vNormal, float3(-.7,  .7, 0.0)))+
                       max(0, dot(In.vNormal, float3( .7, -.7, 0.0))))*
                       g_vDiffuse.rgb + g_vAmbient.rgb;

    return Out;
}


////////////////////////////////////////////////////////////////////////////////
/// \brief Pixel shader for rendering the 3D cursor
////////////////////////////////////////////////////////////////////////////////
PS_OUT CursorPS( VS_OUTCURSOR In )
{ 
    PS_OUT Out;
    Out.RGBColor.rgb = In.Diffuse;
    Out.RGBColor.a   = 1.0f;
    return Out;
}

////////////////////////////////////////////////////////////////////////////////
// Effect technique for rendering the snapping data into the render target
////////////////////////////////////////////////////////////////////////////////
technique SnapRender
{
    pass P0
    {          
        VertexShader = compile vs_1_1 SnapVS();
        PixelShader  = compile ps_2_0 SnapPS();
    }
}

////////////////////////////////////////////////////////////////////////////////
// Effect technique for rendering the actual scene
////////////////////////////////////////////////////////////////////////////////
technique ActualRender
{
    pass P0
    {          
        VertexShader = compile vs_1_1 ActualVS();
        PixelShader  = compile ps_2_0 ActualPS();
    }
}

////////////////////////////////////////////////////////////////////////////////
// Effect technique for rendering the normal vectors
////////////////////////////////////////////////////////////////////////////////
technique NormalRender
{
    pass P0
    {          
        VertexShader = compile vs_1_1 ActualVS();
        PixelShader  = compile ps_2_0 ActualNormalPS();
    }
}

////////////////////////////////////////////////////////////////////////////////
// Effect technique for rendering the normal vectors
////////////////////////////////////////////////////////////////////////////////
technique DepthRender
{
    pass P0
    {          
        VertexShader = compile vs_1_1 SnapVS();
        PixelShader  = compile ps_2_0 DepthPS();
    }
}

////////////////////////////////////////////////////////////////////////////////
// Effect technique for rendering the faces IDs
////////////////////////////////////////////////////////////////////////////////
technique FaceRender
{
    pass P0
    {          
        VertexShader = compile vs_1_1 SnapVS();
        PixelShader  = compile ps_2_0 FacePS();
    }
}

////////////////////////////////////////////////////////////////////////////////
// Effect technique for rendering the barycentric weights
////////////////////////////////////////////////////////////////////////////////
technique BaryRender
{
    pass P0
    {          
        VertexShader = compile vs_1_1 SnapVS();
        PixelShader  = compile ps_2_0 BaryPS();
    }
}

////////////////////////////////////////////////////////////////////////////////
// Effect technique for rendering the 3D cursor
////////////////////////////////////////////////////////////////////////////////
technique CursorRender
{
    pass P0
    {          
        VertexShader = compile vs_1_1 CursorVS();
        PixelShader  = compile ps_1_1 CursorPS();
    }
}
