////////////////////////////////////////////////////////////////////////////////
/// \file snapping.h
///
/// Definition of data structures and state variables used by the sample
/// program that demonstrates snapping to geometry deformed on the GPU.
///
/// (c)2005 Harlen Batagelo, harlen@dca.fee.unicamp.br
////////////////////////////////////////////////////////////////////////////////

#pragma once

////////////////////////////////////////////////////////////////////////////////
/// Vertex declaration for rendering the a model with a normal and a pair
/// of texture coordinates.
////////////////////////////////////////////////////////////////////////////////
struct D3DVERTEX_SIMPLE
{
	D3DXVECTOR3 vP;         ///< Vertex position.
    D3DXVECTOR3 vN;         ///< Vertex normal.
    FLOAT       fU;         ///< First texture coordinate (U). 
	FLOAT       fV;         ///< Second texture coordinate (V).

    static const DWORD FVF;
};

/// Flexible vertex format used for rendering the actual 3D model.
const DWORD D3DVERTEX_SIMPLE::FVF = D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_TEX1 |
							        D3DFVF_TEXCOORDSIZE2(0);

////////////////////////////////////////////////////////////////////////////////
/// Vertex declaration for rendering the actual model with a normal, a pair
/// of texture coordinates, tangent and binormal.
////////////////////////////////////////////////////////////////////////////////
struct D3DVERTEX_ACTUAL
{
	D3DXVECTOR3 vP;         ///< Vertex position.
	D3DXVECTOR3 vN;         ///< Vertex normal.
	FLOAT       fU;         ///< First texture coordinate (U). 
	FLOAT       fV;         ///< Second texture coordinate (V).
	D3DXVECTOR3 vT;         ///< Tangent.
	D3DXVECTOR3 vB;         ///< Binormal.

	static const DWORD FVF;
};

/// Flexible vertex format used for rendering the actual 3D model.
const DWORD D3DVERTEX_ACTUAL::FVF = D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_TEX1 |
									D3DFVF_TEX2 | D3DFVF_TEX3 |
									D3DFVF_TEXCOORDSIZE2(0) |
									D3DFVF_TEXCOORDSIZE3(1) |
									D3DFVF_TEXCOORDSIZE3(2);

////////////////////////////////////////////////////////////////////////////////
/// Vertex declaration for rendering the model with the interaction data.
////////////////////////////////////////////////////////////////////////////////
struct D3DVERTEX_INTERACT
{
    D3DXVECTOR3 vP;         ///< Vertex position.
    D3DXVECTOR3 vN;         ///< Vertex normal.
    FLOAT       fU,         ///< First texture coordinate (U).
	            fV;         ///< Second texture coordinate (V).
	FLOAT       fFace;      ///< Face identifier.
	FLOAT       fBary1,     ///< First barycentric coordinate.
	            fBary2;     ///< Second barycentric coordinate.
	D3DXVECTOR3 vT;         ///< Tangent.
	D3DXVECTOR3 vB;         ///< Binormal.

    static const DWORD FVF;
};

/// Flexible vertex format used for rendering the 3D model for the extended
/// snapping. The model is rendered as a non-indexed vertex stream with
/// additional data for face identification and barycentric coordinates 
/// calculation.
const DWORD D3DVERTEX_INTERACT::FVF = D3DFVF_XYZ  | D3DFVF_NORMAL | 
									  D3DFVF_TEX1 | D3DFVF_TEX2 | 
									  D3DFVF_TEX3 | D3DFVF_TEX4 |
							          D3DFVF_TEXCOORDSIZE2(0) |
							          D3DFVF_TEXCOORDSIZE3(1) |
							          D3DFVF_TEXCOORDSIZE3(2) |
							          D3DFVF_TEXCOORDSIZE3(3);

////////////////////////////////////////////////////////////////////////////////
/// Data structure used by the software snapping algorithm.
////////////////////////////////////////////////////////////////////////////////
struct INTERSECTION
{
    DWORD dwFace;          ///< Mesh face that was intersected.
	FLOAT fX,              ///< X world coordinate of intersection.
		  fY,              ///< Y world coordinate of intersection.
		  fZ,              ///< Z world coordinate of intersection.
		  fW;              ///< W world coordinate of intersection.
	FLOAT fNX,             ///< X normal vector of intersection.
		  fNY,             ///< Y normal vector of intersection.
		  fNZ;             ///< Z normal vector of intersection.
    FLOAT fBary1,          ///< First barycentric coordinate of intersection.
		  fBary2;          ///< Second barycentric coordinate of intersection.
    FLOAT fDist;           ///< Distance from ray origin to intersection.
	D3DXVECTOR3 vTangent;  ///< Tangent vector.
	D3DXVECTOR3 vBinormal; ///< Binormal vector.
};

////////////////////////////////////////////////////////////////////////////////
/// Format of the render target used for rendering the snapping data.
////////////////////////////////////////////////////////////////////////////////
#define RT_FORMAT D3DFMT_A32B32G32R32F


////////////////////////////////////////////////////////////////////////////////
/// \brief Application class.
///
/// The base class (CD3DApplication) provides the generic functionality needed 
/// in all Direct3D samples. CApp adds functionality specific to this sample 
/// program.
////////////////////////////////////////////////////////////////////////////////
class CApp : public CD3DApplication
{
	/// Vertex declaration of the geometry for rendering interaction data.
	LPDIRECT3DVERTEXDECLARATION9 m_pVertexDeclInteract;
	/// Vertex declaration of geometry with a normal and a pair of texture 
	/// coordinates.
	LPDIRECT3DVERTEXDECLARATION9 m_pVertexDeclSimple;
	/// Vertex declaration of the geometry actually rendered.
	LPDIRECT3DVERTEXDECLARATION9 m_pVertexDeclActual;

    CD3DFont*     m_pFont;               ///< Font for drawing text.
    CD3DFont*     m_pFontSmall;          ///< Font for drawing text with a small font.
    
	CD3DMesh*     m_pObject;             ///< Object to render and interact.
	D3DXMATRIXA16 m_mCenter,             ///< Translation matrix to center the object in world space.
				  m_mScale;              ///< Scaling matrix to set the object to a suitable size.
	bool          m_bObjSetUp;           ///< Whether the object was centered and scaled.

    bool          m_bIntersected;        ///< Whether the ray intersected the object.
    INTERSECTION  m_Intersection;        ///< Intersection info.

    LPDIRECT3DVERTEXBUFFER9 m_pVBPicked; ///< VB for picked triangles.
	LPDIRECT3DVERTEXBUFFER9 m_pVB;       ///< Indexed VB for the object to render.
	LPDIRECT3DINDEXBUFFER9  m_pIB;       ///< Index buffer for the object to render.
	LPDIRECT3DVERTEXBUFFER9 m_pVBNI;     ///< Non-indexed VB for the object to render.

	DWORD         m_dwNumVerts;          ///< Number of indexed vertices of the mesh.
	DWORD         m_dwNumFaces;          ///< Number of faces of the mesh.

    BOOL          m_bShowHelp;           ///< Whether to show help.
	bool          m_bShowNormalData;     ///< Whether to show the normal data.
	bool          m_bShowDepthData;      ///< Whether to show the depth data.
	bool          m_bShowFaceID;         ///< Whether to show the faces' IDs.
	bool          m_bShowBaryData;       ///< Whether to show the barycentric weights.
	bool          m_bShowSnapTriangle;   ///< Whether to show the snapped triangle.

	bool          m_bDrawTriad;          ///< Whether to draw the cursor triad.
	D3DXVECTOR3   m_vTriadPos;           ///< World space position of the triad.
	D3DXVECTOR3   m_vTriadNormal;        ///< Normal vector of the triad.

	// Render target variables
	LPDIRECT3DSURFACE9 m_pRTReadSurf;
	LPDIRECT3DSURFACE9 m_pBackBufferSurf;
    LPDIRECT3DTEXTURE9 m_pRTTexture;
	LPDIRECT3DSURFACE9 m_pRTSurface;

	D3DXMATRIXA16 m_mW, m_mV, m_mP, m_mWV, m_mVP, m_mWVP;

	float m_fZNear, // Distance of the near clipping plane
		  m_fZFar;  // Distance of the far clipping plane

	float m_fZDist; // Distance of the viewer from the object

	// D3DX effect interface
    LPD3DXEFFECT m_pEffect;      
	// Parts of the 3D cursor triad to show the picking point
    LPD3DXMESH   m_pTriadCenter, 
				 m_pTriadAxis;   

    D3DXHANDLE   m_hWorld;                    // Handle to var in the effect 
	D3DXHANDLE   m_hView;                     // Handle to var in the effect 
    D3DXHANDLE   m_hWorldViewProjection;      // Handle to var in the effect
	D3DXHANDLE   m_hDiffuse,                  // Handle to var in the effect
				 m_hAmbient;
	D3DXHANDLE   m_hDeform1, m_hDeform2;

    LPDIRECT3DTEXTURE9 m_pTex, m_pBumpTex; // Decal and bump textures

	float m_fDeform1, m_fDeform2; // Parameters used for deforming the object

    // Internal member functions
    HRESULT ConfirmDevice( D3DCAPS9*, DWORD, D3DFORMAT, D3DFORMAT );

	void SetSnapMatrix( int iX, int iY );
	HRESULT SetUpObject( );
	void DrawAt( BYTE *pTexBmp, D3DLOCKED_RECT &rcLockedRect, int iX, int iY, 
			     int iWidth, int iHeight, BYTE bR, BYTE bG, BYTE bB, BYTE bA );
	void MyFrameMove( );
	void UpdateShaderMatrixHandles( );

protected:
    HRESULT OneTimeSceneInit();
    HRESULT InitDeviceObjects();
    HRESULT RestoreDeviceObjects();
    HRESULT InvalidateDeviceObjects();
    HRESULT DeleteDeviceObjects();
    HRESULT Render();
    HRESULT FrameMove();
    HRESULT FinalCleanup();

public:
    LRESULT MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
    CApp();
};